# Piveau Auth Middleware Plugin

Vue plugin to authenticate users against the Piveau ecosystem using Piveau Auth Middleware.

- 🌐 API Client for Piveau Auth Middleware
- 🔒 Automatic management of authentication state
- 🔄 Vue Plugin and composable functions for easy integration into Vue apps
- 🎯 Supports Vue 3 and Vue 2
- 🔧 Vue Devtools integration
- 📜 Typescript support

## Getting Started

Install the plugin using

```bash
npm install @piveau/piveau-ui-plugin-auth
```

### Plugin registration

For Vue 3

```typescript
import { createApp } from 'vue'
import { init } from '@piveau/piveau-ui-plugin-auth'

const { plugin } = init({
  // Options
  baseUrl: 'https://auth.example.com',
  refreshInterval: 1000 * 60 * 5,
  defaultLoginOptions: {
    redirectUrl: 'https://your-app.org/login-redirect',
  }
})

const app = createApp()
app.use(plugin)
```

Vue 2

```typescript
const { plugin: pvAuthPlugin, pvAuth } = init({
  // Options
  baseUrl: 'https://auth.example.com',
  refreshInterval: 1000 * 60 * 5,
  defaultLoginOptions: {
    redirectUrl: 'https://your-app.org/login-redirect',
  }
})

Vue.use(pvAuthPlugin)

new Vue({
  router,
  store,
  pvAuth,
  render: h => h(App),
}).$mount('#app')
```

### Usage

The plugin provides a `pvAuth` object on the Vue instance, which can be used to access the authentication state and perform authentication actions.

```typescript
import { usePvAuth } from '@piveau/piveau-ui-plugin-auth'

const pvAuth = usePvAuth()
if (!pvAuth) return

// Access the authentication state
const isAuthenticated = pvAuth.isAuthenticated.value
```

### Authentication state


**Composition API**

The plugin provides composable functions to access the authentication state. We use tanstack/vue-query in the background to manage the state. Using composition API is recommended.
```typescript
import { defineComponent } from 'vue'
import { useUserInfoQuery, useResourcesQuery, useRptQuery } from '@piveau/piveau-ui-plugin-auth'

export default defineComponent({
  const { data: userInfo } = useUserInfoQuery()
  const { data: resources } = useResourcesQuery()
  const { data: rpt } = useRptQuery()

  return {
    userInfo,
    resources,
    rpt,
  }
})
```

**Options API**

 Alternatively, the $pvAuth object can be used for options API. Remember that composition API and options API can be mixed.

```typescript
import { defineComponent } from 'vue'

export default defineComponent({
  methods: {
    login() {
      this.$pvAuth.login()
    },
    logout() {
      this.$pvAuth.logout()
    },
  },
  created () {
  const authData = this.$pvAuth.authData.value

  const isAuthenticated = authData.isAuthenticated
  const idTokenParsed = authData.idTokenParsed
  const username = authData.username
  const resources = authData.resources
  const rpt = authData.rpt
  }
})
```

**Options API/Composition API**

```typescript
import { defineComponent } from 'vue'

export default defineComponent({
  methods: {
    login() {
      this.$pvAuth.login()
    },
    logout() {
      this.$pvAuth.logout()
    },
  },
  computed: {
    rpt() {
      return this.authData.rpt
    },
  },
  setup() {
    const authData = usePvAuth()

    return {
      authData,
    }
  }
})
```

## Development

Requirements:
- node >= 16.20.2
- npm >= 8.19.4
