import Axios from 'axios'
import { z } from 'zod'
import type { LoginCredentialsParams, LoginEndParams, LoginStartParams, LogoutParams, TokenParams } from '../types/api'
import { schemaLoginCredentials, schemaLoginEnd, schemaLoginStart, schemaLogout, schemaResources, schemaToken, schemaUserinfo } from '../types/api'

type Headers = Record<string, string>

interface RequestOptions {
  headers?: Headers
}

interface ApiRequestOptions<T> {
  apiClient: any
  method: 'post' | 'get'
  endpoint: string
  params?: any
  options?: RequestOptions
  schema: z.Schema<T>
}

async function safeRequestWithSchema<T>({
  apiClient,
  method,
  endpoint,
  params,
  options,
  schema,
}: ApiRequestOptions<T>) {
  const res = await apiClient[method](endpoint, params, {
    headers: {
      ...options?.headers,
    },
  })

  return schema.parse(res.data)
}

export function createPiveauAuthApi(baseUrl: string, options?: {
  headers?: Headers
  apiKey?: string
}) {
  const apiClient = Axios.create({
    baseURL: baseUrl,
    headers: {
      'Content-Type': 'application/json',
      'X-API-Key': options?.apiKey || 'apiKey',
      ...options?.headers,
    },
    withCredentials: true,
  })

  /**
   * Initiates the login procedure for the authorization code flow.
   *
   * Called by the SPA to initiate the login process. Returns the `authorization_request_url`
   * to which the SPA should redirect the user to initiate the code flow. If a session is active
   * (and a user is already logged in), the session is destroyed, all tokens are revoked, the
   * user is logged out, and a new session is created. Some request parameters from
   * [OpenID Connect Core 1.0 Specification](https://openid.net/specs/openid-connect-core-1_0.html#AuthRequest)
   * are allowed (e.g., "ui_locales", "acr_values", "max_age", "display", "prompt").
   *
   * @example
   * ```typescript
   * const piveauAuthApi = createPiveauAuthApi(BASE_URL);
   * const response = await piveauAuthApi.loginStart({
   *   request_parameters: {
   *     ui_locales: "en",
   *     prompt: "login",
   *     display: "page"
   *   },
   *   page_url: "http://example.com/current_location",
   *   redirect_url: "http://example.com/login-redirect"
   * });
   * ```
   */
  const loginStart = async (params: LoginStartParams, options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/login/start',
      params,
      options,
      schema: schemaLoginStart,
    })
  }

  /**
   * Called on every page load with the current URL, which can occur due to:
   * 1. User navigation to its URL.
   * 2. Browser page refresh or new tab opening.
   *
   * @example
   * ```typescript
   * const response = await piveauAuthApi.loginEnd({ current_url: "http://example.com/current_page" });
   * ```
   */
  const loginEnd = async (params: LoginEndParams, options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/login/end',
      params,
      options,
      schema: schemaLoginEnd,
    })
  }

  /**
   * Performs the password flow directly at Keycloak, bypassing EU Login,
   * as EU Login does not currently support this mechanism.
   *
   * @example
   * ```typescript
   * const response = await piveauAuthApi.loginCredentials({ username: "user", password: "password" });
   * ```
   */
  const loginCredentials = async (params: LoginCredentialsParams, options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/login/credentials',
      params,
      options,
      schema: schemaLoginCredentials,
    })
  }

  /**
   * Returns the ID token claims, names, and IDs of accessible catalogs/resources.
   * Resource IDs can be used in the `/token` endpoint to tailor the RPT.
   *
   * @example
   * ```typescript
   * const userInfo = await piveauAuthApi.userinfo();
   * ```
   */
  const userinfo = async (options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'get',
      endpoint: '/userinfo',
      options,
      schema: schemaUserinfo,
    })
  }

  /**
   * Returns resources/catalogs for a given audience, mirroring the structure in the corresponding RPT.
   *
   * @example
   * ```typescript
   * const resources = await piveauAuthApi.resources("audienceValue");
   * ```
   */
  const resources = async (audience?: string, options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/resources',
      params: { audience },
      options,
      schema: schemaResources,
    })
  }

  /**
   * Refreshes EU Login and Keycloak tokens, maintaining both IDP sessions.
   * If a session is invalidated, re-login is required.
   *
   * @example
   * ```typescript
   * const refreshedData = await piveauAuthApi.refresh();
   * ```
   */
  const refresh = async (options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/refresh',
      options,
      schema: z.any(),
    })
  }

  /**
   * Returns the RPT
   */
  const token = async (params?: TokenParams, options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/token',
      params,
      options,
      schema: schemaToken,
    })
  }

  /**
   * Logs the user out.
   */
  const logout = async (params?: LogoutParams, options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/logout',
      params,
      options,
      schema: schemaLogout,
    })
  }

  const bclogout = async (options?: {
    headers?: Headers
  }) => {
    return safeRequestWithSchema({
      apiClient,
      method: 'post',
      endpoint: '/bclogout',
      options,
      schema: z.any(),
    })
  }

  return {
    loginStart,
    loginEnd,
    loginCredentials,
    userinfo,
    resources,
    refresh,
    token,
    logout,
    bclogout,
  }
}

export type PiveauAuthApi = ReturnType<typeof createPiveauAuthApi>
