import type { Ref } from 'vue'
import type { MaybeRefOrGetter } from '@vueuse/core'
import type { QueryClient } from '@tanstack/vue-query'
import { useQuery } from '@tanstack/vue-query'
import { computed, effectScope, ref, unref } from 'vue-demi'
import type { AuthState } from '../types'
import type { PiveauAuthApi } from './api'

export function useAutoRefresh({
  api,
  authData,
  queryClient,
  enableAutoRefresh = ref(true),
  refreshInterval = 10 * 60 * 1000,
  beforeRefresh = () => {},
  afterRefresh = () => {},
  onError = () => {},
}: {
  api: PiveauAuthApi
  authData: AuthState
  queryClient: QueryClient
  enableAutoRefresh?: Ref<boolean>
  refreshInterval?: MaybeRefOrGetter<number>
  beforeRefresh?: () => void
  afterRefresh?: () => void
  onError?: (error: any) => void
}) {
  const scope = effectScope()
  // const { data, error, refetch } = useQuery('refreshToken', api.refresh, {
  //   enabled: false, // Initially disabled, we will refetch manually when needed
  //   retry: false, // If you don't want it to automatically retry on error
  //   refetchInterval: unref(refreshInterval), // Auto refresh interval
  // });

  const localScope = scope.run(() => {
    const enableAutoRefreshResolved = computed(() => {
      return unref(enableAutoRefresh) && authData.isAuthenticated
    })

    const lastUpdated = ref(0)

    const { refetch: refreshToken, isLoading, error } = useQuery({
      queryKey: ['refresh-token'],
      enabled: enableAutoRefreshResolved,
      queryFn: async () => {
        beforeRefresh()
        try {
          const result = await api.refresh()
          afterRefresh()
          lastUpdated.value = Date.now()
          return result
        }
        catch (error) {
          onError(error)
          throw error
        }
      },
      refetchInterval: unref(refreshInterval),
      staleTime: 15 * 1000,
    }, queryClient)

    return {
      refreshToken,
      isLoading,
      error,
      lastUpdated: computed(() => lastUpdated.value),
    }
  })

  if (!localScope)
    throw new Error('No local scope')

  return { ...localScope }
}
