import type { QueryClient } from '@tanstack/vue-query'
import { useStorage } from '@vueuse/core'
import { computed, ref, watch } from 'vue-demi'
import useLoginEndQuery from '../queries/useLoginEndQuery'
import type { PiveauAuth } from '../types'
import { parseRedirectUri, usePvAuth } from '../utils'

export function useLoginRedirectHandler({
  redirectUrl = window.location.href,
  pvAuth = usePvAuth(),
  queryClient,
  onCode,
}: {
  redirectUrl?: string
  pvAuth?: PiveauAuth
  queryClient?: QueryClient
  onCode?: (code: string | string[]) => void
}) {
  if (!pvAuth)
    throw new Error('No PiveauAuth instance provided.')

  const api = pvAuth.apiClient
  const qc = queryClient ?? pvAuth.options.queryClient!
  const csrf = useStorage('piveau-hub-ui-plugin-auth--csrf', '')
  const enabled = computed(() => pvAuth.options.enabled)
  const query = useLoginEndQuery({
    enabled,
    api,
    csrf,
    pageUrl: window.location.href,
    qc,
  })

  const code = ref('')
  const state = ref('')
  const iss = ref('')

  /**
   * Starts the login redirect handler.
   * @param cb The callback function that is called when the user is logged in.
   * @returns A function to cancel the handler.
   */
  function start(cb: (url: string) => void) {
    const isLoggedIn = computed(() => !!query.data.value?.is_logged_in)
    let cancel = () => {}
    let loggedInOnce = false

    cancel = watch(isLoggedIn, (_isLoggedIn) => {
      if (_isLoggedIn && !loggedInOnce) {
        loggedInOnce = true
        const { oauthParams, newUri } = parseRedirectUri(redirectUrl)
        const _code = oauthParams.code
        if (_code) {
          onCode?.(_code)
          window.history.replaceState(history.state, '', newUri)
        }

        code.value = typeof oauthParams.code === 'string' ? oauthParams.code : ''
        state.value = typeof oauthParams.state === 'string' ? oauthParams.state : ''
        iss.value = typeof oauthParams.iss === 'string' ? oauthParams.iss : ''

        if (pvAuth) {
          pvAuth.redirectParams.code = code.value
          pvAuth.redirectParams.state = state.value
          pvAuth.redirectParams.iss = iss.value
        }

        if (query.data.value?.stored_page_url)
          cb(query.data.value?.stored_page_url)

        cancel()
      }
    }, {
      immediate: true,
    })
    return cancel
  }

  return {
    oathParams: {
      code,
      state,
      iss,
    },
    query,
    start,
  }
}
