/**
 * @fileoverview Manages the authentication state and provides methods to login, logout, and refresh the authentication data.
 */

import type { MaybeRef } from 'vue-demi'
import { computed, effectScope, reactive, ref, watchEffect } from 'vue-demi'
import { QueryClient } from '@tanstack/vue-query'
import { useStorage, useTimestamp, whenever } from '@vueuse/core'
import { isAxiosError } from 'axios'
import { type AuthState, schemaUserinfo } from '../types'
import useLoginEndQuery from '../queries/useLoginEndQuery'
import { useRptQuery } from '../queries/useRptQuery'
import { useResourcesQuery } from '../queries/useResourcesQuery'
import { useUserinfoQuery } from '../queries/useUserInfoQuery'
import { decodeJwt } from '../utils'
import type { PiveauAuthApi } from './api'
import { useAutoRefresh } from './autoRefresh'

export const authData = reactive<AuthState>({
  isAuthenticated: false,
  idTokenParsed: null,
  username: null,
  resources: null,
  rpt: null,
})

export function resetAuthData() {
  authData.isAuthenticated = false
  authData.idTokenParsed = null
  authData.username = null
  authData.resources = null
  authData.rpt = null
}

/**
 * Sets up the authentication manager with the provided configuration.
 *
 */
export function setup({
  api,
  refreshInterval,
  queryClient,
  enabled = true,
}: {
  api: PiveauAuthApi
  refreshInterval: number
  queryClient?: QueryClient
  enabled?: MaybeRef<boolean>
}) {
  const qc = queryClient ?? new QueryClient()
  resetAuthData()
  const scope = effectScope()
  const enableAutoRefresh = ref(true)

  const authScope = scope.run(() => {
    const { refreshToken, error, lastUpdated: lastAutoRefreshed } = useAutoRefresh({
      api,
      authData,
      enableAutoRefresh,
      refreshInterval,
      beforeRefresh: () => {},
      onError: (error) => {
        console.error('Error during auto-refresh:', error)
      },
      queryClient: qc,
    })

    // Only enable rpt, resources, userinfo if auto-refresh was successful
    const enabledQueries = computed(() => !!lastAutoRefreshed.value)

    // Invalidate queries whenever auto-refresh is successful
    whenever(lastAutoRefreshed, () => {
      qc.invalidateQueries({ queryKey: ['rpt'] })
      qc.invalidateQueries({ queryKey: ['resources'] })
      qc.invalidateQueries({ queryKey: ['userinfo'] })
    })

    const rptQuery = useRptQuery({ api, enabled: enabledQueries, qc })
    const resourcesQuery = useResourcesQuery({ api, enabled: enabledQueries, qc })
    const userinfoQuery = useUserinfoQuery({ api, enabled: enabledQueries, qc })

    const csrf = useStorage('piveau-hub-ui-plugin-auth--csrf', '')
    const loginQuery = useLoginEndQuery({
      enabled,
      api,
      qc,
      csrf,
      pageUrl: window.location.href,
    })

    watchEffect(() => {
      if (authData.isAuthenticated) {
        authData.rpt = rptQuery.data.value?.access_token || null
        authData.resources = resourcesQuery.data.value || null
        authData.idTokenParsed = userinfoQuery.data.value || null
        authData.username
          = userinfoQuery.data.value?.preferred_username || null
      }
    })

    watchEffect(() => {
      if (loginQuery.data.value?.csrf)
        csrf.value = loginQuery.data.value.csrf

      if (loginQuery.data.value?.is_logged_in)
        authData.isAuthenticated = true
    })

    const isRefreshErrorOr401 = computed(() => {
      if (isAxiosError(rptQuery.error.value))
        return rptQuery.error.value.response?.status === 401
      return error.value
    })

    whenever(isRefreshErrorOr401, () => {
      resetAuthData()
      authData.isAuthenticated = false
    })

    const rptExpiresIn = computed(() => {
      if (!rptQuery.isSuccess.value)
        return Number.NaN

      const now = useTimestamp({
        interval: 1000,
      })

      const expirationDate = (userinfoQuery.data.value?.session_expires_in || Number.NaN) * 1000 + lastAutoRefreshed.value

      if (!expirationDate)
        return Number.NaN

      return expirationDate - now.value
    })

    async function refreshAuthData() {
      await rptQuery.refetch()
      await resourcesQuery.refetch()
      await userinfoQuery.refetch()

      authData.isAuthenticated = true
    }

    return {
      refreshToken,
      lastRefreshed: computed(() => lastAutoRefreshed.value),
      authData: computed(() => authData),
      rptQuery,
      useLoginEndQuery,
      resourcesQuery,
      userinfoQuery,
      refreshAuthData,
      rptExpiresIn,
    }
  })

  if (!authScope)
    throw new Error('rootScope is undefined')

  if (typeof window !== 'undefined')
    qc.mount()

  return {
    queryClient: qc,
    enableAutoRefresh,
    ...authScope!,
  }
}
