import type { QueryClient } from '@tanstack/vue-query'
import type { App, InjectionKey } from 'vue-demi'
import { computed, install, isVue2, markRaw, reactive, ref } from 'vue-demi'
import { isAxiosError } from 'axios'
import { createPiveauAuthApi } from './auth/api'
import { setup } from './auth/manager'
import { createNavigationGuard } from './auth/navigationGuard'
import type { OauthRequestParameters, PiveauAuth, PluginOptions } from './types'
import { schemaInitOptions } from './types'
import { NavigateToLoginPageOptions, navigateToLoginPage } from './auth/navigateToLoginPage'
import { navigateToLogoutPage } from './auth/navigateToLogoutPage'
import { setupDevtools } from './devtools'

if (isVue2)
  install()

export { useLoginRedirectHandler } from './auth/loginRedirectHandler'
export { authData } from './auth/manager'
export { default as LoginRedirect } from './components/LoginRedirect'
export { default as useLoginEndQuery } from './queries/useLoginEndQuery'
export { useResourcesQuery } from './queries/useResourcesQuery'
export { useRptQuery } from './queries/useRptQuery'
export { useUserinfoQuery } from './queries/useUserInfoQuery'
export * from './utils'
export * from './types'
export * from './globalExtensions'

export const key = Symbol('') as InjectionKey<PiveauAuth>
export const keyQueryClient = Symbol('') as InjectionKey<QueryClient>

export function definePiveauAuthOptions(options: PluginOptions) {
  return options
}

export function init(options: PluginOptions) {
  const opts = schemaInitOptions.parse(options)
  const { baseUrl, queryClient, refreshInterval } = opts
  const api = createPiveauAuthApi(baseUrl)

  if (!opts.enabled)
    console.warn('[PiveauAuth] Plugin is disabled.')

  const enabled = ref(opts.enabled)

  const {
    authData,
    enableAutoRefresh,
    rptQuery,
    resourcesQuery,
    queryClient: qc,
    refreshToken,
    lastRefreshed,
    refreshAuthData,
    rptExpiresIn,
  } = setup({
    api,
    refreshInterval,
    queryClient,
    enabled,
  })

  const pvAuth: PiveauAuth = {
    login: (options) => {
      const {
        redirectUrl,
        requestParameters,
      } = options || {}
      return navigateToLoginPage({
        api,
        redirectUrl: redirectUrl || opts.defaultLoginOptions?.redirect_url,
        requestParameters: requestParameters || opts.defaultLoginOptions?.requestParameters,
      }).catch((e) => {
        if (isAxiosError(e)) {
          if (e.response?.status === 400)
            console.error(`[PiveauAuth] Login URL retrieval failed. Have you made sure that the provided redirect URL (${redirectUrl || opts.defaultLoginOptions?.redirect_url}) is registered in the Piveau Auth server?`, e)
          else
            console.error('[PiveauAuth] Login URL retrieval failed:', e)
        }
        throw e
      })
    },
    logout: () => navigateToLogoutPage({ api, pageUrl: opts.defaultLoginOptions?.redirect_url, redirectUrl: opts.defaultLogoutOptions?.post_logout_redirect_url }),
    setEnabled: (v: boolean) => enabled.value = v,
    authData,
    refreshToken,
    refreshAuthData,
    autoRefresh: enableAutoRefresh,
    lastRefreshed,
    redirectParams: reactive({
      code: undefined,
      state: undefined,
      iss: undefined,
    }),
    options: {
      ...opts,
      queryClient: qc,
    },
    cache: {
      rpt: rptQuery,
      resources: resourcesQuery,
    },
    rptExpiresIn,
    apiClient: api,
  }

  const plugin = markRaw({
    install(app: App) {
      if (isVue2) {
        app.mixin({
          beforeCreate() {
            const options = this.$options

            if (options.pvAuth) {
              const _pv = options.pvAuth

              if (!(this as any)._provided) {
                const provideCache = {}
                Object.defineProperty(this, '_provided', {
                  get: () => provideCache,
                  set: (v) => {
                    Object.assign(provideCache, v)
                  },
                })
              }
              (this as any)._provided[key as any] = _pv

              if (!this.$pvAuth)
                this.$pvAuth = pvAuth

              setupDevtools(this, pvAuth)
            }
            else if (
              !this.$pvAuth
              && options.parent
              && options.parent.$pvAuth
            ) {
              this.$pvAuth = options.parent.$pvAuth
            }
          },
        })
      }
      else {
        app.provide(keyQueryClient, qc)
        app.provide(key, pvAuth)
        app.config.globalProperties.$pvAuth = pvAuth
        setupDevtools(app, pvAuth)
      }
    },
  })

  const navigationGuardFn = createNavigationGuard({
    enabled,
    api,
    onRequiredAuth: _api => navigateToLoginPage({ api: _api, redirectUrl: opts.defaultLoginOptions?.redirect_url }),
    isLoggedIn: computed(() => authData.value.isAuthenticated),
    qc,
  })

  return {
    plugin,
    pvAuth,
    navigationGuardFn,
  }
}
