import { z } from 'zod'

/**
 * Partial OAuth AuthRequest parameters.
 *
 * @see https://openid.net/specs/openid-connect-core-1_0.html#AuthRequest
 */
export const schemaOauthRequestParameters = z.object({
  /**
   * End-User's preferred languages and scripts for the user interface, represented as a
   * space-separated list of BCP47 [RFC5646] language tag values, ordered by preference.
   *
   * @example 'en-US de-DE'
   */
  ui_locales: z.string().optional(),

  /**
   * Requested Authentication Context Class Reference values.
   *
   * @example 'https://ecas.ec.europa.eu/loa/basic'
   */
  acr_values: z.string().optional(),

  /**
   * ASCII string value that specifies whether the Authorization Server prompts the End-User
   * for reauthentication and consent.
   *
   * @example 'login'
   */
  prompt: z.enum(['none', 'login', 'consent', 'select_account']).optional(),

  /**
   * ASCII string value that specifies how the Authorization Server displays the authentication
   *
   * @example 'page'
   */
  display: z.enum(['page', 'popup', 'touch', 'wap']).optional(),
})

/**
 * The parameters for the login start endpoint.
 */
export const schemaLoginStartParams = z.object({
  requestParameters: schemaOauthRequestParameters,

  /**
   * This parameter is utilized to save and recover the current location (state) before the login page, as EU Login does not permit dynamic redirect URIs.
   *
   * @example 'http://localhost:3030/current_location'
   */
  page_url: z.string().url(),

  /**
   * Redirect Url after login page. no dynamic parts allowed, have to be configured at the middleware and OIDC client. Is used as post logout redirect uri.
   *
   * @example: 'http://localhost:3030/login-redirect'
   */
  redirect_url: z.string().url(),

  /**
   * The preferred languages and scripts for the user interface.
   */
  ui_locales: z.string().optional(),
})

export const schemaLoginEndParams = z.object({
  /**
   * This parameter is utilized to save and recover the current location (state) before the login page, as EU Login does not permit dynamic redirect URIs.
   *
   * @example 'http://localhost:3030/current_location'
   */
  page_url: z.string().url(),
})

export const schemaLoginCredentialsParams = z.object({
  /**
   * The username.
   */
  username: z.string(),

  /**
   * The password.
   */
  password: z.string(),
})

export const schemaLogoutParams = z.object({
  /**
   * Restores your current location after redirect from logout page. EU login does not allow dynamic redirect uris after login and logout, so please use this parameter. This url is returned after page load and invocation of login/end.
   */
  page_url: z.string().url(),

  /**
   * Post logout redirect URL. Has to be configured in IDP, if necessary. if not given, the redirect_url from login/start call is used
   */
  post_logout_redirect_url: z.string().url().optional(),

  /**
   * Can be used to control, whether the endsession url (aka logout url) is called by the middleware or not. If not, the frontend can, but do not have to call the logout_url. The frontend can also call the logout_url if it was already called by the middleware
   */
  call_endsession_url: z.boolean().default(false),
})

export const schemaTokenParams = z.object({
  /**
   * Keycloak specific permissions
   *
   * @see https://www.keycloak.org/docs/latest/authorization_services/index.html#_service_obtaining_permissions
   * @example ["f688a84b-d874-4e07-9e33-e615b763cac3","63cb263b-7e58-48f2-8324-780ee2a090ab"]
   */
  permissions: z.array(z.string()).optional(),

  /**
   * The target audience for the token.
   */
  audience: z.string().optional(),

  /**
   * The IDP to use for authentication.
   *
   * @example 'eulogin'
   */
  idp: z.string().optional(),
})

export const schemaLoginStart = z.object({
  authorization_request_url: z.string().url(),
})

export const schemaLoginEnd = z.object({
  is_logged_in: z.boolean(),
  stored_page_url: z.string().url().nullable().optional(),
  error: z.string().optional(),
  error_description: z.string().optional(),
  csrf: z.string().optional(),
})

export const schemaLoginCredentials = z.object({
  accessToken: z.string(),
  refreshToken: z.string(),
  expiresIn: z.number(),
  tokenType: z.string(),
  scope: z.string(),
})

export const schemaScopes = z.enum([
  'dataset:update',
  'dataset:delete',
  'dataset:create',
  'catalogue:update',
  'catalogue:delete',
  'catalogue:create',
])

export const schemaResources = z.object({
  permissions: z.array(
    z.object({
      scopes: z.array(schemaScopes),
      rsid: z.string(),
      rsname: z.string(),
    }),
  ).optional(),
})

export const schemaUserinfo = z.object({
  email_verified: z.boolean(),
  address: z.object({}).passthrough().optional(),
  name: z.string(),
  preferred_username: z.string(),
  given_name: z.string(),
  family_name: z.string(),
  email: z.string().email(),
  session_expires_in: z.number(),
}).partial().passthrough()

export const schemaToken = z.object({
  access_token: z.string(),
})

export const schemaLogout = z.object({
  logout_url: z.string().url(),
})

export type LoginStartResponse = z.infer<typeof schemaLoginStart>
export type LoginEndResponse = z.infer<typeof schemaLoginEnd>
export type LoginCredentialsResponse = z.infer<typeof schemaLoginCredentials>
export type LogoutResponse = z.infer<typeof schemaLogout>
export type ResourcesResponse = z.infer<typeof schemaResources>
export type UserinfoResponse = z.infer<typeof schemaUserinfo>
export type TokenResponse = z.infer<typeof schemaToken>

export type LoginStartParams = z.infer<typeof schemaLoginStartParams>
export type LoginEndParams = z.infer<typeof schemaLoginEndParams>
export type LoginCredentialsParams = z.infer<typeof schemaLoginCredentialsParams>
export type LogoutParams = z.infer<typeof schemaLogoutParams>
export type TokenParams = z.infer<typeof schemaTokenParams>

export type OauthRequestParameters = z.infer<typeof schemaOauthRequestParameters>
