import type { ComputedRef, Ref } from 'vue-demi'
import { z } from 'zod'
import type { UseQueryReturnType } from '@tanstack/vue-query'
import type {
  PiveauAuthApi,
} from '../auth/api'
import type { NavigateToLoginPageOptions } from '../auth/navigateToLoginPage'
import {
  schemaLoginStartParams,
  schemaLogoutParams,
  type schemaResources,
  type schemaToken,
} from './api'

export * from './api'

/**
 * The plugin options.
 *
 * @example ```ts
 * {
 *  baseUrl: 'https://auth.example.com',
 *  refreshInterval: 600000,
 * }
 */
export const schemaInitOptions = z.object({
  /**
   * Whether to enable authentication on startup
   * @default true
   */
  enabled: z.boolean().default(true),

  /**
   * The base URL of the Piveau Auth server.
   */
  baseUrl: z.string(),

  /**
   * The refresh interval in milliseconds.
   * @default 600000
   */
  refreshInterval: z
    .number()
    .int()
    .gte(-1)
    .default(10 * 60 * 1000),

  /**
   * The query client to use. If not provided, a new one will be created.
   * @see https://tanstack.com/query/v4/docs/vue/overview
   */
  queryClient: z.any(),

  /**
   * The default parameters for the login method.
   */
  defaultLoginOptions: schemaLoginStartParams.partial().optional(),

  /**
   * The default parameters for the logout method.
   */
  defaultLogoutOptions: schemaLogoutParams.partial().optional(),
})

export type InitOptions = z.infer<typeof schemaInitOptions>
export type PluginOptions = Partial<InitOptions>

/**
 * The authentication state.
 */
export interface AuthState {
  isAuthenticated: boolean
  idTokenParsed: any | null
  username: string | null
  resources: z.infer<typeof schemaResources> | null
  rpt: string | null
}

export type LoginOptions = Omit<NavigateToLoginPageOptions, 'api'>

/**
 * The PiveauAuth instance.
 */
export interface PiveauAuth {
  /**
   * Fetches the login URL and redirects the user agent to it.
   */
  login(options?: LoginOptions): Promise<void>

  /**
   * Fetches the logout URL and redirects the user agent to it.
   */
  logout(): Promise<void>

  /**
   * Sets whether automatic token refresh is enabled.
   */
  setEnabled(enabled: boolean): void

  /**
   * Refreshes the authentication data.
   */
  refreshToken(): void

  /**
   * Refreshes the authentication data.
   */
  refreshAuthData(): Promise<void>

  /**
   * The authentication state.
   */
  authData: ComputedRef<AuthState>

  /**
   * Instance options.
   */
  options: z.infer<typeof schemaInitOptions>

  /**
   * The last time the authentication data was refreshed.
   */
  lastRefreshed: ComputedRef<number>

  /**
   * The redirect parameters from OAuth callback
   */
  redirectParams: {
    code: string | undefined
    state: string | undefined
    iss: string | undefined
  }

  autoRefresh: Ref<boolean>
  cache: {
    rpt: UseQueryReturnType<z.infer<typeof schemaToken>, unknown>
    resources: UseQueryReturnType<z.infer<typeof schemaResources>, unknown>
  }

  rptExpiresIn: ComputedRef<number | null>

  apiClient: PiveauAuthApi
}

/**
 * Route location compatible with vue-router 3.x and 4.x.
 */
export interface RouteLocationLike {
  path: string
  query: Record<string, string | string[] | undefined>
  hash: string
  matched: any[]
}
