import { inject } from 'vue-demi'
import type { QueryClient } from '@tanstack/vue-query'
import type { RouteLocationLike } from './types'
import type { PiveauAuthApi } from './auth/api'
import { key } from '.'

const SUPPORTED_PARAMS = ['code', 'state', 'iss'] as const

export function typeSafeIncludes<T extends U, U>(
  arr: readonly T[],
  searchElement: U,
): searchElement is T {
  return (arr as readonly any[]).includes(searchElement)
}

function parseRedirectParams(paramsString: string) {
  const params = new URLSearchParams(paramsString)
  const oauthParams: {
    [key in (typeof SUPPORTED_PARAMS)[number]]?: string | string[];
  } = {}
  const otherParams: {
    [key: string]: string | string[]
  } = {}

  for (const [key, value] of params.entries()) {
    if (typeSafeIncludes(SUPPORTED_PARAMS, key))
      oauthParams[key] = value
    else
      otherParams[key] = value
  }

  return { oauthParams, otherParams }
}

/**
 * Parses the redirect URI and returns an object containing the OAuth parameters, other parameters, redirect URI, and new URI without OAuth parameters.
 */
export function parseRedirectUri(url: string) {
  const parsedUrl = new URL(url)
  const { oauthParams, otherParams } = parseRedirectParams(
    parsedUrl.search.slice(1),
  )

  // generate newUri without oauthParams
  const newUri = new URL(parsedUrl.origin + parsedUrl.pathname)
  for (const [key, value] of Object.entries(otherParams))
    newUri.searchParams.set(key, value as string)

  // Restore fragment
  newUri.hash = parsedUrl.hash

  return {
    /**
     * The OAuth parameters parsed from the redirect URI.
     */
    oauthParams,
    /**
     * The other parameters parsed from the redirect URI.
     */
    otherParams,
    /**
     * The redirect URI without any query parameters.
     */
    redirectUri: parsedUrl.origin + parsedUrl.pathname,
    /**
     * The new URI without the OAuth parameters.
     */
    newUri: newUri.toString(),
  }
}

/**
 * Resolves a route to an href.
 */
export function resolveRouteToHref({
  baseUrl = window.location.href,
  to,
}: {
  baseUrl?: string
  to: RouteLocationLike
}) {
  const url = new URL(baseUrl)
  url.pathname = to.path || '/'
  url.hash = to.hash || ''
  return url.toString()
}

/**
 * Resolves an href to a route.
 */
export function resolveHrefToRoute({
  baseUrl = window.location.href,
  href,
}: {
  baseUrl?: string
  href: string
}) {
  const url = new URL(href, baseUrl)
  return {
    path: url.pathname,
    hash: url.hash,
  }
}

/**
 * Returns the current PiveauAuth instance.
 */
export function usePvAuth() {
  const pvAuth = inject(key)

  return pvAuth
}

interface ResolveOptions {
  api?: PiveauAuthApi
  qc?: QueryClient
}

export function useApiAndClient(options?: ResolveOptions): { api: PiveauAuthApi, qc: QueryClient } {
  const resolvedApi = options?.api || usePvAuth()?.apiClient
  const resolvedQueryClient = options?.qc || usePvAuth()?.options.queryClient

  if (!resolvedApi)
    throw new Error('no api provided')

  if (!resolvedQueryClient)
    throw new Error('no queryClient provided')

  return { api: resolvedApi, qc: resolvedQueryClient }
}

export function decodeJwt(token: string = '') {
  if (!token)
    return
  return JSON.parse(atob(token.split('.')?.[1] || ''))
}
