import { afterEach, describe, expect, it, vi } from 'vitest'
import { computed } from 'vue'
import { QueryClient } from '@tanstack/vue-query'
import { createPiveauAuthApi } from '../src/auth/api'
import { createNavigationGuard } from '../src/auth/navigationGuard'

// Mock the API module
vi.mock('../src/auth/api', () => ({
  createPiveauAuthApi: vi.fn(() => ({
    loginEnd: vi.fn(() => ({ is_logged_in: true, stored_page_url: undefined })),
    // ... other methods, if needed
  })),
}))

describe('createNavigationGuard', () => {
  afterEach(() => {
    vi.clearAllMocks()
  })

  it('exports', () => {
    expect(createNavigationGuard).toBeDefined()
  })

  function setupMocks({
    is_logged_in,
    stored_page_url = null,
  }: {
    is_logged_in
    stored_page_url?: null
  }) {
    const mockedApi = createPiveauAuthApi('someBaseUrl')
    mockedApi.loginEnd = vi.fn(() =>
      Promise.resolve({ is_logged_in, stored_page_url }),
    )
    const onRequiredAuth = vi.fn()
    const next = vi.fn()
    return { mockedApi, onRequiredAuth, next }
  }

  async function executeNavigationGuard({
    enabled = true,
    requiresAuth,
    mockedApi,
    onRequiredAuth,
    next,
    is_logged_in,
  }: {
    enabled?
    requiresAuth
    mockedApi
    onRequiredAuth?
    next?
    is_logged_in?
  }) {
    const navigationGuard = createNavigationGuard({
      enabled,
      api: mockedApi,
      onRequiredAuth,
      isLoggedIn: computed(() => !!is_logged_in),
      qc: new QueryClient({
        defaultOptions: {
          queries: { gcTime: Number.POSITIVE_INFINITY, retry: false },
        },
      }),
    })
    const to = { matched: [{ meta: { requiresAuth } }] }
    const from = {}
    await navigationGuard(to, from, next)
  }

  it('should passthrough if route does not require auth', async () => {
    const { mockedApi, onRequiredAuth, next } = setupMocks({
      is_logged_in: false,
    })
    await executeNavigationGuard({
      requiresAuth: false,
      mockedApi,
      onRequiredAuth,
      next,
    })
    expect(next).toHaveBeenCalled()
    expect(onRequiredAuth).not.toHaveBeenCalled()
  })

  it('should call api.loginEnd if route requires auth', async () => {
    const { mockedApi, onRequiredAuth, next } = setupMocks({
      is_logged_in: true,
    })
    await executeNavigationGuard({
      requiresAuth: true,
      mockedApi,
      onRequiredAuth,
      next,
    })
    expect(mockedApi.loginEnd).toHaveBeenCalled()
  })

  it('should redirect if user is not logged in', async () => {
    const { mockedApi, onRequiredAuth, next } = setupMocks({
      is_logged_in: false,
    })
    await executeNavigationGuard({
      requiresAuth: true,
      mockedApi,
      onRequiredAuth,
      next,
    })
    expect(onRequiredAuth).toHaveBeenCalled()
    expect(next).not.toHaveBeenCalled()
  })

  it('should passthrough if user is logged in', async () => {
    const { mockedApi, next, onRequiredAuth } = setupMocks({
      is_logged_in: true,
    })
    await executeNavigationGuard({
      requiresAuth: true,
      mockedApi,
      onRequiredAuth,
      next,
      is_logged_in: true,
    })

    expect(onRequiredAuth).not.toHaveBeenCalled()
    expect(next).toHaveBeenCalled()
  })

  it('if enabled is false, should not call api.loginEnd', async () => {
    const { mockedApi, onRequiredAuth, next } = setupMocks({
      is_logged_in: true,
    })
    await executeNavigationGuard({
      requiresAuth: true,
      enabled: false,
      mockedApi,
      onRequiredAuth,
      next,
    })
    expect(mockedApi.loginEnd).not.toHaveBeenCalled()
    // next called with false
    expect(next).toHaveBeenCalledWith(false)
  })
})
