import type { Ref } from 'vue-demi'
import { nextTick, reactive, ref } from 'vue-demi'
import type { Mock } from 'vitest'
import { afterEach, beforeEach, describe, expect, it, vi } from 'vitest'
import { QueryClient } from '@tanstack/vue-query'
import { useAutoRefresh } from '../src/auth/autoRefresh'

import { createPiveauAuthApi } from '../src/auth/api'

// Change to the actual path to the module that exports your PiveauAuthApi
import type { AuthState } from '../src/types'

// Mock the API module
vi.mock('../src/auth/api', () => ({
  createPiveauAuthApi: vi.fn(() => ({
    refresh: vi.fn(() => Promise.resolve({ foo: 'bar' })), // Mocking refresh method
    // ... other methods, if needed
  })),
}))

describe('useAutoRefresh', () => {
  let mockedApi: ReturnType<typeof createPiveauAuthApi>
  let authData = {} as AuthState
  let beforeRefresh: Mock
  let afterRefresh: Mock
  let onError: Mock
  let enableAutoRefresh: Ref<boolean>
  let refreshToken: any
  let queryClient = {} as QueryClient

  beforeEach(() => {
    queryClient = new QueryClient({
      defaultOptions: {
        queries: {
          retry: false,
          cacheTime: Number.POSITIVE_INFINITY,
        },
      },
    })
    mockedApi = createPiveauAuthApi('someBaseUrl')
    authData = reactive<AuthState>({
      isAuthenticated: true,
      idTokenParsed: null,
      username: null,
      resources: null,
      rpt: null,
    })
    beforeRefresh = vi.fn()
    afterRefresh = vi.fn()
    onError = vi.fn()
    enableAutoRefresh = ref(true);
    ({ refreshToken } = useAutoRefresh({
      api: mockedApi,
      authData,
      refreshInterval: 100000,
      enableAutoRefresh,
      beforeRefresh,
      afterRefresh,
      onError,
      queryClient,
    }))

    vi.useFakeTimers()
  })

  afterEach(() => {
    queryClient.unmount()
    vi.clearAllMocks()
  })

  it('exports', () => {
    expect(useAutoRefresh).toBeDefined()
  })

  it('refreshes', async () => {
    const mockedApi = createPiveauAuthApi('someBaseUrl')

    const authData = reactive({
      isAuthenticated: false,
      idTokenParsed: null,
      username: null,
      resources: null,
      rpt: null,
    })

    const beforeRefresh = vi.fn()
    const afterRefresh = vi.fn()
    const onError = vi.fn()

    const { refreshToken } = useAutoRefresh({
      api: mockedApi,
      authData,
      beforeRefresh,
      afterRefresh,
      onError,
      queryClient: new QueryClient({
        defaultOptions: {
          queries: {
            retry: false,
            cacheTime: Number.POSITIVE_INFINITY,
          },
        },
      }),
    })

    expect(mockedApi.refresh).to.not.toHaveBeenCalled()

    vi.mocked(mockedApi.refresh).mockResolvedValue({ foo: 'bar' })
    await refreshToken()

    // expect(refreshSignal.value).toBe('queuedManual');
    expect(mockedApi.refresh).toHaveBeenCalledTimes(1)
    expect(beforeRefresh).toHaveBeenCalledTimes(1)
    expect(afterRefresh).toHaveBeenCalledTimes(1)
  })

  it('refreshes after interval', async () => {
    vi.mocked(mockedApi.refresh).mockResolvedValue({ foo: 'bar' })

    await vi.advanceTimersByTimeAsync(150000)
    await nextTick()

    // expect(refreshSignal.value).toBe('queuedAuto');
    expect(mockedApi.refresh).toHaveBeenCalledTimes(2)
    expect(beforeRefresh).toHaveBeenCalledTimes(2)
    expect(afterRefresh).toHaveBeenCalledTimes(2)
  })

  it('refreshes periodically', async () => {
    vi.mocked(mockedApi.refresh).mockResolvedValue({ foo: 'bar' })

    await vi.advanceTimersByTimeAsync(150000)
    await nextTick()

    expect(mockedApi.refresh).toHaveBeenCalledTimes(2)
    expect(beforeRefresh).toHaveBeenCalledTimes(2)
    expect(afterRefresh).toHaveBeenCalledTimes(2)

    await vi.advanceTimersByTimeAsync(100000)
    await nextTick()

    expect(mockedApi.refresh).toHaveBeenCalledTimes(3)
    expect(beforeRefresh).toHaveBeenCalledTimes(3)
    expect(afterRefresh).toHaveBeenCalledTimes(3)
  })

  it('stops refreshing when auto refresh is disabled', async () => {
    vi.mocked(mockedApi.refresh).mockResolvedValue({ foo: 'bar' })

    await vi.advanceTimersByTimeAsync(150000)
    await nextTick()

    expect(mockedApi.refresh).toHaveBeenCalledTimes(2)
    expect(beforeRefresh).toHaveBeenCalledTimes(2)
    expect(afterRefresh).toHaveBeenCalledTimes(2)

    enableAutoRefresh.value = false

    await vi.advanceTimersByTimeAsync(100000)
    await nextTick()

    expect(mockedApi.refresh).toHaveBeenCalledTimes(2)
    expect(beforeRefresh).toHaveBeenCalledTimes(2)
    expect(afterRefresh).toHaveBeenCalledTimes(2)
  })

  it('handles errors', async () => {
    vi.mocked(mockedApi.refresh).mockRejectedValue(new Error('Some error'))
    await refreshToken()
    await nextTick()
  })
})
